#!/bin/sh

#
# Copyright (C) 2012 Carsten Presser <c@rstenpresser.de>
#
# This file is part of karate_tools
#
# karate_tools is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# karate_tools is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with fsusb_picdem; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA
#
#

# Device name of the KarateLight device
KL_DEV="/dev/ttyACM0"

# you might want to change these :)
BIN_KARATE_ENTER_BOOT=`which karate-enter-boot`
BIN_USBRESET=`which usbreset`
BIN_FSUSB=`which fsusb`
BIN_KARATE_GETINFO=`which karate-getinfo`

# base url for getting the firmware
URL_BASE="http://ca.rstenpresser.de/tl_files/data/firmware/karate/"

# USB IDs for the KarateLight device
KL_USB_VENDOR="04d8"
KL_USB_ID_BOOT="000b"
KL_USB_ID_CDC="000a"

# locaction of the usb-dev-fs
USBDEVFS="/dev/bus/usb"

#
LOGFILE="update-firmware.log"


# Color definitions
CYAN="\033[0;36m"
RED="\033[0;31m"
GREEN="\033[0;32m"
NO_COLOR="\033[0m"


log_success() {
  echo $GREEN[okay]$NO_COLOR $1
}

log_info() {
  echo $CYAN[info]$NO_COLOR $1
}

cleanup() {
  rm -rf $TMP
  cd $PP
}

log_error() {
  echo $RED[fail]$NO_COLOR $1
  if [ -e $LOGFILE ]; then
    log_info "------------- start of logfile ---------------------"
    cat $LOGFILE
    log_info "--------------- end of logfile ---------------------"
  fi;
  cleanup
  exit 1
}

log_warning() {
  echo $RED[warn]$NO_COLOR $1
}


#
# check for interpreter, disable colors
#
if [ `ls -la /bin/sh | grep busybox | wc -l` -eq 1 ]; then
  CYAN=""
  RED=""
  GREEN=""
  NO_COLOR=""
  log_info "Busybox shell detected: disabling colors."
fi;


log_info "`basename $0` Version 20120530"

#
# Parse Commandline
#
while [ $# -gt 0 ]; do
  if [ "$1" = "testing" ] || [ "$1" = "stable" ] || [ "$1" = "experimental" ]; then
    URL_FW="karate_v2."$1".hex"
    URL_MD5=$URL_FW".md5"
    shift
    continue
  fi;
  if [ "`echo $1 | cut -b 1-5`" = "/dev/" ]; then
    KL_DEV=$1
    log_info "operating with device $KL_DEV"
    shift
    continue
  fi
  log_warning "unrecognized parameter: "$1
  shift
done;

if [ a$URL_FW = "a" ]; then
  log_error "firmware-version not specified. please add stable, testing or experimental to the commandline"
fi;

#
# Check all binarys..
#
if [ ! -x $BIN_KARATE_GETINFO ]; then
  log_error "karate-getinfo binary not found"
fi
if [ ! -x $BIN_FSUSB ]; then
  log_error "fsusb binary not found"
fi
if [ ! -x $BIN_USBRESET ]; then
  log_error "usbreset binary not found"
fi
if [ ! -x $BIN_KARATE_ENTER_BOOT ]; then
  log_error "karate-enter-boot binary not found"
fi



#
# only run as root
#
if [ "$USER" != "root" ]; then
  log_error "I am not the superuser.. please use sudo or su"
fi;

#
# Prepare Temp Dir
#
PP=`pwd` 
TMP=`mktemp -d /tmp/tmp.XXXXXX`
cd $TMP

#
# Make sure not other process is writing to the device
#
if [ `ps | grep atmo | grep -v grep | wc -l` -gt 0 ]; then
  if [ -e /etc/init.d/atmolight ]; then
    /etc/init.d/atmolight stop > $LOGFILE
    log_warning "atmolight daemon was still running!"
  else
    log_error "Shut down the atmolight daemon first"
  fi;
fi;

#
# search usbdevfs
#
if [ -e /proc/bus/usb/devices ]; then
  USBDEVFS="/proc/bus/usb"
  log_warning "Found deprecated usb-fileystem /proc/bus/usb"
fi;


#
# make sure we can access the usb-filesystem
#
if [ ! -d $USBDEVFS ]; then
  log_error "USB filesystem $USBDEVFS not found!"
fi;

#
# Download Firmware Files
#
log_info "Trying to download "$URL_BASE/$URL_FW
wget -q -P $TMP $URL_BASE/$URL_FW
RET1=$?
log_info "Trying to download "$URL_BASE/$URL_MD5
wget -q -P $TMP $URL_BASE/$URL_MD5
RET2=$?

if [ $RET1 -ne 0 ] || [ $RET2 -ne 0 ]; then
  log_error "At least one download failed. Check you Internet connection and make sure the URL is correct"
fi;

# check md5sum
md5sum -c $TMP/$URL_MD5 > $LOGFILE
if [ $? -ne 0 ]; then
  log_error "md5sum verification failed. try downloading the files again."
else
  log_success "download of Firmware-files complete."
fi;


#
# Search KarateLight
#
KL=`lsusb | grep -n $KL_USB_VENDOR`
if [ "a$KL" = "a" ]; then
  log_error "KarateLight device not found. Please check if lsusb is working"
fi;

# Reset the device..
KL_USB_BUS=`echo $KL | cut -d " " -f 2`
KL_USB_DEV=`echo $KL | cut -d " " -f 4 | cut -b 1-3`
KL_PROC_PATH=$USBDEVFS/$KL_USB_BUS/$KL_USB_DEV
$BIN_USBRESET $KL_PROC_PATH > $LOGFILE
if [ $? != "0" ]; then
  log_error "failed to reset the KarateLight device"
fi;

# search again...
sleep 1
KL=`lsusb | grep -n $KL_USB_VENDOR`


# check for cdc-mode
KL_CDC=`echo $KL | grep -n $KL_USB_ID_CDC`
if [ $? -ne 0 ]; then 
  log_error "KarateLight device not in CDC-mode! Please power-cycle the KarateLight hardware."
fi;

# check if device is writable
if [ ! -w $KL_DEV ]; then
  log_error "Device \""$KL_DEV"\" is not writable. Check the device-name and permissions."
fi;

# in case we need usbreset
KL_USB_BUS=`echo $KL | cut -d " " -f 2`
KL_USB_DEV=`echo $KL | cut -d " " -f 4 | cut -b 1-3`
KL_PROC_PATH=$USBDEVFS/$KL_USB_BUS/$KL_USB_DEV


#
# Enter Boot-Mode
#
log_info "trying to enter boot mode"
$BIN_KARATE_ENTER_BOOT $KL_DEV > $LOGFILE
if [ $? -ne 0 ]; then 
  log_error "Running karate-enter-boot failed"
fi;
sleep 1

# check if device has been re-enumerated...
if [ -e $KL_PROC_PATH ]; then
  $BIN_USBRESET $KL_PROC_PATH > $LOGFILE
fi;
sleep 1

# check for success
KL=`lsusb | grep -n $KL_USB_VENDOR`
KL_BOOT=`echo $KL | grep -n $KL_USB_ID_BOOT`
if [ $? -ne 0 ]; then 
  log_error "Cant find a KarateLight device in BOOT-mode! Try replugging USB without a power cylcle.\n        If this does not help, you might want to try the Enter-Bootload switch found inside the Case."
fi;
log_success "Device is now in Boot mode"

#
# Now Flash the Firmware
#
$BIN_FSUSB --program $TMP/$URL_FW > $LOGFILE
if [ $? -ne 0 ]; then
  log_error "Writing the Firmware failed!!"
fi;
log_success "Writing the firmware was successfull"

$BIN_FSUSB --reset > $LOGFILE
if [ $? -ne 0 ]; then
  log_error "Reseting the device failed! Just try a power-cylcle."
fi;
sleep 1

# in case we need usbreset
KL_USB_BUS=`echo $KL | cut -d " " -f 2`
KL_USB_DEV=`echo $KL | cut -d " " -f 4 | cut -b 1-3`
KL_PROC_PATH=$USBDEVFS/$KL_USB_BUS/$KL_USB_DEV

# check if device has been re-enumerated...
if [ -e $KL_PROC_PATH ]; then
  $BIN_USBRESET $KL_PROC_PATH > $LOGFILE
fi;

sleep 1

# check if device is back in CDC mode...
KL=`lsusb | grep -n $KL_USB_VENDOR`
if [ "a$KL" = "a" ]; then
  log_error "KarateLight device not found. Just try a power-cycle."
fi;

# check for cdc-mode
KL_CDC=`echo $KL | grep -n $KL_USB_ID_CDC`
if [ $? -ne 0 ]; then 
  log_error "KarateLight device not in CDC Mode. Just try a power-cycle."
fi;

log_success "Updating the firmware was successfull"

#
# get infos and display them
#
KINFO=`$BIN_KARATE_GETINFO $KL_DEV`;
KRINFORET=$?;

if [ $KRINFORET -ne 0 ]; then
  log_error "running karate_getinfo failed"
  exit
fi

KRFW=`echo $KINFO | cut  -d" " -f 1`;
log_info "Firmware-Version now is $KRFW"

KRBY=`echo $KINFO | cut  -d" " -f 2`;
KRCH=$(($KRBY * 8 / 12 / 3))

log_info "Device reports $KRCH Channels"


#
# Cleanup
#
cleanup
